from _typeshed import Incomplete

def cmp(a: int, b: int) -> int: ...

class PythonVersion:
    version: Incomplete
    def compare(self, expected_str: str) -> int: ...
    def __eq__(self, expected: object) -> bool: ...
    def __lt__(self, expected: str) -> bool: ...
    def __le__(self, expected: str) -> bool: ...
    def __gt__(self, expected: str) -> bool: ...
    def __ge__(self, expected: str) -> bool: ...

PY: Incomplete
